import {readdir, readFile, writeFile} from 'node:fs/promises';
import * as path from 'node:path';

let error = false;
let album = '';

const root = process.cwd();
const topNames = await readdir(root);
const folders = [];

album +=
  `Album: Generated Album\n`;

for (const topName of topNames) {
  if (topName === '.DS_Store') continue;
  if (topName === 'album.yaml') continue;

  if (topName.match(/^[0-9]+ /)) {
    console.log(` * ${topName}`);

    folders.push(topName)
    continue;
  }

  console.error(` ? ${topName}`);
  error = true;
}

if (error) {
  console.error(`Encountered unexpected names, see above.`);
  console.error(`Make sure to run this from a directory where`);
  console.error(`the only contents are numbered folders.`);
  process.exit(1);
}

folders.sort();

for (const [sectionIndex, folder] of folders.entries()) {
  console.log(`-- ${folder} --`);

  album +=
    `---\n` +
    `Section: ${folder.match(/[0-9]+ (.*)/)[1]}\n`;

  const files = await readdir(path.join(root, folder));
  const tracks = [];
  const texts = [];

  let warnUnknown = false;
  let warnMissingText = false;

  for (const file of files) {
    if (file.endsWith('.txt')) {
      if (files.includes(file.slice(0, -4))) continue;

      console.error(` ? ${file}`);
      warnUnknown = true;
      continue;
    }

    if (file.endsWith('.mp3')) {
      tracks.push(file);
      if (files.includes(file + '.txt')) {
        texts.push(
          await readFile(
            path.join(root, folder, file + '.txt'),
            'utf8'));
        console.log(` * ${file}`);
      } else {
        texts.push(null);
        console.warn(` ! ${file}`);
        console.warn(`   (missing text file)`);
        warnMissingText = true;
      }
      continue;
    }
  }

  if (warnUnknown) {
    console.warn(`Some files in this directory weren't expected.`);
    console.warn(`They'll be skipped/ignored.`);
  }

  if (warnMissingText) {
    console.warn(`Some tracks in this directory lack corresponding`);
    console.warn(`text files. They won't have that data included in YAML.`);
  }

  for (const [trackIndex, track] of tracks.entries()) {
    const text = texts[trackIndex];

    const title =
      text?.match(/^ *title *: (.*)/m);

    const name =
      (title
        ? title[1]
        : track);

    const duration =
      text?.match(/^ *Duration: ([0-9:]*)/m);

    album +=
      `---\n` +
      `Track: ${JSON.stringify(name.trim())}\n` +
      `Directory: st-YEAR-${1+sectionIndex}-${1+trackIndex}\n` +
      `Always Reference By Directory: true\n`;

    if (duration) {
      album +=
        `Duration: ${duration[1].replace(/^00:0?/, '')}\n`;
    }

    if (text) {
      album +=
        `Commentary: |-\n` +
        `    <i>Homestuck Sound Test:</i>\n` +
        `    *Filename:* <code>${track}</code>\n` +
        `    <blockquote class="metadata">\n` +
        text
          .replace(/[]/g, '') // the evil
          .trim()
          .split('\n')
          .map((l, i, {length}) =>
            `    ` +
            l
              .replace(/^ */g, m => '&nbsp;'.repeat(m.length))
              .replace(/ *$/g, '')
            + (i === length - 1 ? '\n' : '<br>\n'))
          .join('') +
        `    </blockquote>\n`;
    }
  }
}

await writeFile('album.yaml', album, 'utf8');
